'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var precondition = require('./precondition.js');
var defaultEncoding = require('./default-encoding.js');
var sync = require('./sync.js');
var toBuffer = require('./to-buffer.js');

var toBrowser = {
    'sha': 'SHA-1',
    'sha-1': 'SHA-1',
    'sha1': 'SHA-1',
    'sha256': 'SHA-256',
    'sha-256': 'SHA-256',
    'sha384': 'SHA-384',
    'sha-384': 'SHA-384',
    'sha-512': 'SHA-512',
    'sha512': 'SHA-512',
};
var nextTick;
function getNextTick() {
    if (nextTick) {
        return nextTick;
    }
    if (global.process && global.process.nextTick) {
        nextTick = global.process.nextTick;
    }
    else if (global.queueMicrotask) {
        nextTick = global.queueMicrotask;
    }
    else if (global.setImmediate) {
        nextTick = global.setImmediate;
    }
    else {
        nextTick = global.setTimeout;
    }
    return nextTick;
}
function resolvePromise(promise, callback) {
    promise.then(function (out) {
        getNextTick()(function () {
            callback(null, out);
        });
    }, function (e) {
        getNextTick()(function () {
            callback(e, Buffer.alloc(0));
        });
    });
}
function async(password, salt, iterations, keylen, digest, callback) {
    if (typeof digest === 'function') {
        callback = digest;
        digest = undefined;
    }
    digest = digest || 'sha1';
    var algo = toBrowser[digest.toLowerCase()];
    if (!algo || typeof global.Promise !== 'function') {
        getNextTick()(function () {
            var out;
            try {
                out = sync.sync(password, salt, iterations, keylen, digest);
            }
            catch (e) {
                return callback(e, Buffer.alloc(0));
            }
            callback(null, out);
        });
        return;
    }
    precondition.checkParameters(iterations, keylen);
    password = toBuffer.toBuffer(password, defaultEncoding.defaultEncoding, 'Password');
    salt = toBuffer.toBuffer(salt, defaultEncoding.defaultEncoding, 'Salt');
    if (typeof callback !== 'function')
        throw new Error('No callback provided to pbkdf2');
    resolvePromise(Promise.resolve(sync.sync(password, salt, iterations, keylen, digest)), callback);
}

exports.async = async;
